import os
import pysam
import pandas as pd
import csv
from typing import Tuple
import math

from run_adni_with_dnabert import get_gene_location
import motif_utils

def seq2kmer(seq, k):
    """
    Convert original sequence to kmers
    
    Arguments:
    seq -- str, original sequence.
    k -- int, kmer of length k specified.
    
    Returns:
    kmers -- str, kmers separated by space

    """
    kmer = [seq[x:x+k] for x in range(len(seq)+1-k)]
    kmers = " ".join(kmer)
    return kmers


def extract_full_sequence_from_fasta(fasta_filename: str) -> str:
    """
    From the given FASTA file, get the full nucleotide sequence as a string.

    Makes string all uppercase.
    """
    ref = pysam.FastaFile(fasta_filename)
    seq = ref.fetch()
    return seq.upper()

def generate_tsv_for_window_UKBB(
    AD_diagnosis_filepath: str,
    family_history_filepath: str,
    fasta_directory: str,
    output_tsv_filepath: str,
    tokenize: bool,
    loglevel: bool = False,
) -> None:
    """
    Create a TSV file with the DNA sequence as first column, and label
    of "AD", "FH AD" (family history of AD), or "No AD" as second column for all FASTA 
    files listed in case/control file, with labels as indicated in this file.
    """
    directory = os.fsencode(fasta_directory)
    family_history_df = pd.read_csv(family_history_filepath,
                             sep=" ", usecols=["ID_1", "MATERNAL_MARIONI", "PATERNAL_MARIONI"])
    AD_diagnosis_df = pd.read_csv(AD_diagnosis_filepath)
    print(family_history_df["MATERNAL_MARIONI"].unique())
    print(family_history_df["PATERNAL_MARIONI"].unique())
    num_failed_files = 0 

    with open(output_tsv_filepath, "w+") as tsvfile:
        writer = csv.writer(tsvfile, delimiter="\t")

        for file in os.listdir(directory):
            filename = os.fsdecode(file)
            if filename.endswith(".fa"): 
                patient_id = int(filename[:7])

                # get label for patient
                label = "No AD"
                if patient_id in AD_diagnosis_df["ID1"].values:
                    label = "AD"
                else:
                    if patient_id not in family_history_df["ID_1"].values:
                        # label = "No AD"
                        print("patient not in family hist df")
                        pass
                    else:
                        col_names = ["MATERNAL_MARIONI", "PATERNAL_MARIONI"]
                        for i in range(len(col_names)):
                            fam_hist_val = family_history_df[family_history_df["ID_1"] == patient_id]["MATERNAL_MARIONI"].iat[0,0]
                            if math.isnan(fam_hist_val):
                                label = "No AD"
                                print("patient not in family hist df")
                            elif int(fam_hist_val) == 1:
                                label = "FH AD"
                                break

                print(label)
                    # print(family_history_df.head())
                    # print(family_history_df[family_history_df["ID_1"] == patient_id].head())
                    # maternal_hist = int(family_history_df[family_history_df["ID_1"] == patient_id]["MATERNAL_MARIONI"].iat[0,0])
                    # paternal_hist = int(family_history_df[family_history_df["ID_1"] == patient_id]["PATERNAL_MARIONI"].iat[0,0])

                    # maternal_hist = int(family_history_df.loc[str(patient_id), "MATERNAL_MARIONI"])
                    # paternal_hist = int(family_history_df.loc[str(patient_id), "PATERNAL_MARIONI"])
                    # if maternal_hist == 1 or paternal_hist == 1: 
                        # label = "FH AD"

                # get FASTA file name and read DNA sequence from file, tokenize if necessary,
                # and write line to output file
    #             fasta_filename = fasta_directory + patient_id + "_0_0_APOE.fa"
    #             try:
    #                 dna_seq = extract_full_sequence_from_fasta(fasta_filename)
    #                 if not tokenize:
    #                     writer.writerow([dna_seq, label])
    #                 else: 
    #                     kmer = seq2kmer(dna_seq, 6)
    #                     writer.writerow([kmer, label])
    #             except OSError as not_found:
    #                 num_failed_files += 1
    #                 if loglevel: 
    #                     print(fasta_filename, "not found; continuing")

    #             if progress % 50 == 0:
    #                 if loglevel:
    #                     print("progress:", progress, "lines done")
    #             progress += 1

    # print("Number of failed files:", num_failed_files)


def main():
    # AD_DIAGNOSIS_FILE = "/mnt/LVM_38TB/UKBB/Bulk_1_labels/AD_Diagnosis.csv"
    # FAMILY_HISTORY_FILE = "/mnt/LVM_38TB/UKBB/Bulk_1_labels/Variables_UKB.txt"
    # FASTA_DIRECTORY = "/mnt/LVM_38TB/UKBB/Bulk_1_APOE_FASTA"
    AD_DIAGNOSIS_FILE = "/home/user/UKBB_data/AD_Diagnosis.csv"
    FAMILY_HISTORY_FILE = "/home/user/UKBB_data/Variables_UKB.txt"
    FASTA_DIRECTORY = "/home/user/UKBB_data"
    generate_tsv_for_window_UKBB(AD_DIAGNOSIS_FILE, FAMILY_HISTORY_FILE, FASTA_DIRECTORY,
            "/home/user/UKBB_data/output.tsv", False
            )

    # windows = generate_start_end_windows(gene_locs, window_size=100)
    # print(windows)
    # fasta_file = "/home/user/fasta1/ADNI_phased_chr19_941_S_4420.fa"

    # window = get_window_at_loc(44908684, 100)

    # generate_tsv_for_window(
    #     window,
    #     "/home/user/ADNI_case_control.txt",
    #     "/home/user/DNABERT_model/data/output.tsv",
    #     "/home/user/fasta1/",
    # )

    # split_tsv_into_test_train(
    #     "/home/user/DNABERT_model/data/output.tsv", "/home/user/DNABERT_model/data/train.tsv",
    #     "/home/user/DNABERT_model/data/dev.tsv", 0.9
    # )

    # run_model_with_multiple_windows(windows, "/home/user/DNABERT/examples/run_finetune.py", "/home/user/DNABERT_model/data")

    # introduce_artificial_signal("/home/user/DNABERT_model/data/output.tsv",
    #                             "/home/user/DNABERT_model/data/artificial_signal_10/output.tsv",
    #                             0.1)
    # split_tsv_into_test_train(
    #     "/home/user/DNABERT_model/data/artificial_signal_10/output.tsv",
    #     "/home/user/DNABERT_model/data/artificial_signal_10/train.tsv",
    #     "/home/user/DNABERT_model/data/artificial_signal_10/dev.tsv", 0.9
    # )


if __name__ == "__main__":
    main()